/*
 * Copyright (c) 2019, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/***** Includes *****/
/* Standard C Libraries */
#include <stdlib.h>
#include <unistd.h>

/* TI Drivers */
#include <ti/drivers/rf/RF.h>
#include <ti/drivers/PIN.h>
#include <ti/drivers/pin/PINCC26XX.h>
#include <ti/drivers/I2C.h>


/* Driverlib Header files */
#include DeviceFamily_constructPath(driverlib/rf_prop_mailbox.h)

/* Board Header files */
#include "hdc2010.h"

/* Board Header files */
#include "ti_drivers_config.h"
#include <ti_radio_config.h>

 I2C_Transaction i2cTransaction;

//#include "HDC20X0.h"
extern HDC2010_Config HDC2010_config[];

/***** Defines *****/

/* Do power measurement */
#define POWER_MEASUREMENT

/* Packet TX Configuration */
#define PAYLOAD_LENGTH      6
#ifdef POWER_MEASUREMENT

// Adjust timeout here!!!
#define PACKET_INTERVAL     10  /* For testing set packet interval to XX seconds, here 10 */

#else
#define PACKET_INTERVAL     500000  /* Set packet interval to 500000us or 500ms */
#endif

#define HDC_SENSOR 0
uint8_t HDC_data[4];
static uint8_t seqNumber = 0;


/***** Prototypes *****/

/***** Variable declarations *****/
static RF_Object rfObject;
static RF_Handle rfHandle;

/* Pin driver handle */
static PIN_Handle ledPinHandle;
static PIN_State ledPinState;

static uint8_t packet[PAYLOAD_LENGTH];
static uint8_t seqNumber;

/*
 * Application LED pin configuration table:
 *   - All LEDs board LEDs are off.
 */
PIN_Config pinTable[] =
{
 //   Board_PIN_LED1 | PIN_GPIO_OUTPUT_EN | PIN_GPIO_LOW | PIN_PUSHPULL | PIN_DRVSTR_MAX,
#ifdef POWER_MEASUREMENT
#if defined(Board_CC1350_LAUNCHXL)
    Board_DIO30_SWPWR | PIN_GPIO_OUTPUT_EN | PIN_GPIO_HIGH | PIN_PUSHPULL | PIN_DRVSTR_MAX,
#endif
#endif
    PIN_TERMINATE
};

/***** Function definitions *****/

void *mainThread(void *arg0)
{
    // initialize optional I2C bus parameters
    uint8_t         rfCycle = 1;
    uint8_t         txBuffer[2];
    uint8_t         rxBuffer[4];
    uint16_t manId = 0;
    uint16_t devId = 0;
    I2C_Handle      i2c;
    I2C_Params      i2cParams;
    I2C_Transaction i2cTransaction;
    i2cTransaction.slaveAddress = HDC2010_SA1; //0x40

    RF_Params rfParams;
    RF_Params_init(&rfParams);

    // One-time init of I2C driver
    GPIO_init();
    I2C_init();

    /* Configure the DIO pins */
    GPIO_setConfig(CONFIG_GPIO_24, GPIO_CFG_OUT_STD | GPIO_CFG_OUT_LOW);
    /* 3 ms reset time for the HDC2010 */
    sleep(2);
    GPIO_setConfig(CONFIG_GPIO_26, GPIO_CFG_OUT_STD | GPIO_CFG_OUT_HIGH);
    GPIO_setConfig(CONFIG_GPIO_7, GPIO_CFG_OUT_STD | GPIO_CFG_OUT_HIGH);

    /* Create I2C for usage */
    I2C_Params_init(&i2cParams); // Default params values
    i2cParams.bitRate = I2C_400kHz;  // modify data rate
    i2cParams.transferCallbackFxn = NULL;
    i2cParams.transferMode = I2C_MODE_BLOCKING;

    i2c = I2C_open(CONFIG_I2C_0, &i2cParams);
    if (i2c == NULL) {
    //  Display_printf(display, 0, 0, "Error Initializing I2C\n");
        while (1);
    }

    /* Common I2C transaction setup */
    i2cTransaction.writeBuf   = txBuffer;
    i2cTransaction.writeCount = 1;
    i2cTransaction.readBuf    = rxBuffer;
    i2cTransaction.readCount  = 4;

    // Read out ManID
    txBuffer[0] = HDC2010_MANUFACTURER_ID_L_REG;  /*MAN ID */
    if (I2C_transfer(i2c, &i2cTransaction)) {
               /* Fixed values in HDC2010, see datasheet  */
               manId = (rxBuffer[0] << 8) | (rxBuffer[1]);
               devId = (rxBuffer[2] << 8) | (rxBuffer[3]);
    }
    // Configure HDC2010 to measure: Okay
    i2cTransaction.writeCount = 2; // 2 Bytes
    i2cTransaction.readCount  = 0;
    txBuffer[0] = HDC2010_RST_DRDY_INT_CONF_REG;
    //txBuffer[1] = 0x56; // measure every 1 sec
    //txBuffer[1] = 0x26; // measure every 60 sec
    txBuffer[1] = 0x36; // measure every 10 sec;
    I2C_transfer(i2c, &i2cTransaction);

    // Configure HDC2010 to measure
    i2cTransaction.writeCount = 2; // 2 Bytes
    i2cTransaction.readCount  = 0;
    txBuffer[0] = HDC2010_MEAS_CONFIG_REG;
    txBuffer[1] = 0x01; // START measurement
    I2C_transfer(i2c, &i2cTransaction);
    GPIO_write(CONFIG_GPIO_7, 0);
    I2C_close(i2c);

    /* Open LED pins */
    ledPinHandle = PIN_open(&ledPinState, pinTable);
    if (ledPinHandle == NULL)
    {
        while(1);
    }

#ifdef POWER_MEASUREMENT
#if defined(Board_CC1350_LAUNCHXL)
    /* Route out PA active pin to Board_DIO30_SWPWR */
    PINCC26XX_setMux(ledPinHandle, Board_DIO30_SWPWR, PINCC26XX_MUX_RFC_GPO1);
#endif
#endif

    RF_cmdPropTx.pktLen = PAYLOAD_LENGTH;
    RF_cmdPropTx.pPkt = packet;
    RF_cmdPropTx.startTrigger.triggerType = TRIG_NOW;

    /* Request access to the radio */
#if defined(DeviceFamily_CC26X0R2)
    rfHandle = RF_open(&rfObject, &RF_prop_2gfsk50kbps_0, (RF_RadioSetup*)&RF_cmdPropRadioSetup, &rfParams);
#else
    rfHandle = RF_open(&rfObject, &RF_prop, (RF_RadioSetup*)&RF_cmdPropRadioDivSetup, &rfParams);
#endif// DeviceFamily_CC26X0R2

    /* Set the frequency */
    RF_postCmd(rfHandle, (RF_Op*)&RF_cmdFs, RF_PriorityNormal, NULL, 0);

    while(1)
    {
       i2c = I2C_open(CONFIG_I2C_0, &i2cParams);
           if (i2c == NULL) {
           //    Display_printf(display, 0, 0, "Error Initializing I2C\n");
                while (1);
           }

        /* Create packet with incrementing sequence number and HDC2010 data payload */
        packet[0] = (uint8_t)(seqNumber >> 8);
        packet[1] = (uint8_t)(seqNumber++);

        /* Common I2C transaction setup */
        i2cTransaction.writeBuf   = txBuffer;
        i2cTransaction.writeCount = 1;
        i2cTransaction.readBuf    = rxBuffer;
        i2cTransaction.readCount  = 4;

        // Read out Temp. and Humidity
        txBuffer[0] = HDC2010_TEMPERATURE_LSB_REG;

        GPIO_write(CONFIG_GPIO_7, 1); //PullUps enabled

        if (I2C_transfer(i2c, &i2cTransaction)) {
          GPIO_write(CONFIG_GPIO_7, 0); //PullUps disabled
          I2C_close(i2c);
          packet[2] = rxBuffer[1];
          packet[3] = rxBuffer[0];
          packet[4] = rxBuffer[3];
          packet[5] = rxBuffer[2];
        }

        if(rfCycle == 6) { // TX after 6 x 10 sec = 1 minute
             /* Send packet */
             RF_EventMask terminationReason = RF_runCmd(rfHandle, (RF_Op*)&RF_cmdPropTx,
                                                   RF_PriorityNormal, NULL, 0);
             rfCycle = 0;

             switch(terminationReason)
             {
                case RF_EventLastCmdDone:
                    // A stand-alone radio operation command or the last radio
                    // operation command in a chain finished.
                    break;
                case RF_EventCmdCancelled:
                    // Command cancelled before it was started; it can be caused
                // by RF_cancelCmd() or RF_flushCmd().
                    break;
                case RF_EventCmdAborted:
                    // Abrupt command termination caused by RF_cancelCmd() or
                    // RF_flushCmd().
                    break;
                case RF_EventCmdStopped:
                    // Graceful command termination caused by RF_cancelCmd() or
                    // RF_flushCmd().
                    break;
                default:
                    // Uncaught error event
                    while(1);
            }

        uint32_t cmdStatus = ((volatile RF_Op*)&RF_cmdPropTx)->status;
        switch(cmdStatus)
        {
            case PROP_DONE_OK:
                // Packet transmitted successfully
                break;
            case PROP_DONE_STOPPED:
                // received CMD_STOP while transmitting packet and finished
                // transmitting packet
                break;
            case PROP_DONE_ABORT:
                // Received CMD_ABORT while transmitting packet
                break;
            case PROP_ERROR_PAR:
                // Observed illegal parameter
                break;
            case PROP_ERROR_NO_SETUP:
                // Command sent without setting up the radio in a supported
                // mode using CMD_PROP_RADIO_SETUP or CMD_RADIO_SETUP
                break;
            case PROP_ERROR_NO_FS:
                // Command sent without the synthesizer being programmed
                break;
            case PROP_ERROR_TXUNF:
                // TX underflow observed during operation
                break;
            default:
                // Uncaught error event - these could come from the
                // pool of states defined in rf_mailbox.h
                while(1);
        }
     }

        rfCycle++;

#ifndef POWER_MEASUREMENT
        PIN_setOutputValue(ledPinHandle, Board_PIN_LED1,!PIN_getOutputValue(Board_PIN_LED1));
#endif
        /* Power down the radio */
        RF_yield(rfHandle);

#ifdef POWER_MEASUREMENT
        /* Sleep for PACKET_INTERVAL s */
        sleep(PACKET_INTERVAL);
#else
        /* Sleep for PACKET_INTERVAL us */
        usleep(PACKET_INTERVAL);
#endif
    }
}
